@@ -9,7 +9,7 @@
  * in the Software without restriction, including without limitation the rights
  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following condition:
+ * furnished to do so, subject to the following conditions:
  * 
  * The above copyright notice and this permission notice shall be included in
  * all copies or substantial portions of the Software.
@@ -164,19 +164,20 @@ void crypto_init() {
 
 }
 
-/* returns 0 if we have a match for algo, -1 otherwise */
+/* returns DROPBEAR_SUCCESS if we have a match for algo, DROPBEAR_FAILURE
+ * otherwise */
 int have_algo(char* algo, int algolen, algo_type algos[]) {
 
 	int i = 0;
 	while (algos[i].name != NULL) {
 		if (strlen(algos[i].name) == algolen
 				&& (strncmp(algos[i].name, algo, algolen) == 0)) {
-			return 0;
+			return DROPBEAR_SUCCESS;
 		}
 		i++;
 	}
 
-	return -1;
+	return DROPBEAR_FAILURE;
 }
 
 
