@@ -52,6 +52,7 @@ static void authclear() {
 	ses.authstate.authdone = 0;
 	ses.authstate.pw = NULL;
 	ses.authstate.username = NULL;
+	ses.authstate.printableuser = NULL;
 	ses.authstate.authtypes = 0;
 #ifdef DROPBEAR_PUBKEY_AUTH
 	ses.authstate.authtypes |= AUTH_TYPE_PUBKEY;
@@ -167,32 +168,39 @@ out:
 static int checkusername(unsigned char *username, unsigned int userlen) {
 
 	char* shell;
+	char* newprintableuser;
 	
 	TRACE(("enter checkusername"));
 	if (userlen > MAX_USERNAME_LEN) {
 		return DROPBEAR_FAILURE;
 	}
 
+	newprintableuser = stripcontrol(username);
+
 	/* new user or username has changed */
 	if (ses.authstate.username == NULL ||
 		strcmp(username, ses.authstate.username) != 0) {
 			/* the username needs resetting */
 			if (ses.authstate.username != NULL) {
-				dropbear_log(LOG_AUTHPRIV | LOG_WARNING,
-						"client trying multiple usernames: '%s' and '%s'",
-						ses.authstate.username, username);
+				dropbear_log(LOG_WARNING,
+					"client trying multiple usernames: '%s' and '%s' from %s",
+					ses.authstate.printableuser, newprintableuser,
+					ses.addrstring);
 				m_free(ses.authstate.username);
+				m_free(ses.authstate.printableuser);
 			}
 			authclear();
 			ses.authstate.pw = getpwnam((char*)username);
 			ses.authstate.username = strdup(username);
+			ses.authstate.printableuser = newprintableuser;
 	}
 
 	/* check that user exists */
 	if (ses.authstate.pw == NULL) {
 		TRACE(("leave checkusername: user doesn't exist"));
-		dropbear_log(LOG_AUTHPRIV | LOG_WARNING,
-				"login attempt for nonexistant user '%s'", username);
+		dropbear_log(LOG_WARNING,
+				"login attempt for nonexistant user '%s' from %s",
+				username, ses.addrstring);
 		send_msg_userauth_failure(0, 1);
 		return DROPBEAR_FAILURE;
 	}
@@ -200,8 +208,9 @@ static int checkusername(unsigned char *username, unsigned int userlen) {
 	/* check for an empty password */
 	if (ses.authstate.pw->pw_passwd[0] == '\0') {
 		TRACE(("leave checkusername: empty pword"));
-		dropbear_log(LOG_AUTHPRIV | LOG_WARNING,
-				"disallowing login for '%s' - empty password", username);
+		dropbear_log(LOG_WARNING,
+				"disallowing login for '%s' from %s - empty password",
+				username, ses.addrstring);
 		send_msg_userauth_failure(0, 1);
 		return DROPBEAR_FAILURE;
 	}
@@ -223,8 +232,9 @@ static int checkusername(unsigned char *username, unsigned int userlen) {
 	/* no matching shell */
 	endusershell();
 	TRACE(("no matching shell"));
-	dropbear_log(LOG_AUTHPRIV | LOG_WARNING,
-			"disallowing login for '%s' - invalid shell", username);
+	dropbear_log(LOG_WARNING,
+			"disallowing login for '%s' from %s - invalid shell",
+			username, ses.addrstring);
 	send_msg_userauth_failure(0, 1);
 	return DROPBEAR_FAILURE;
 	
@@ -286,7 +296,7 @@ void send_msg_userauth_failure(int partial, int incrfail) {
 		if (ses.authstate.username == NULL) {
 			userstr = "is unknown!!!!";
 		} else {
-			userstr = ses.authstate.username;
+			userstr = ses.authstate.printableuser;
 		}
 		dropbear_exit("Max auth tries reached - user %s", userstr);
 	}
