@@ -112,12 +112,14 @@ void pubkeyauth() {
 	/* ... and finally verify the signature */
 	if (buf_verify(ses.payload, key, buf_getptr(signbuf, signbuf->len),
 				signbuf->len) == DROPBEAR_SUCCESS) {
-		dropbear_log(LOG_AUTHPRIV | LOG_INFO,
-				"pubkey auth succeeded for '%s'", ses.authstate.username);
+		dropbear_log(LOG_NOTICE,
+				"pubkey auth succeeded for '%s' from %s",
+				ses.authstate.printableuser, ses.addrstring);
 		send_msg_userauth_success();
 	} else {
-		dropbear_log(LOG_AUTHPRIV | LOG_INFO,
-				"pubkey auth failure for '%s'", ses.authstate.username);
+		dropbear_log(LOG_WARNING,
+				"pubkey auth failure for '%s' from %s",
+				ses.authstate.printableuser, ses.addrstring);
 		send_msg_userauth_failure(0, 1);
 	}
 
@@ -170,26 +172,28 @@ static int checkpubkey(unsigned char* algo, unsigned int algolen,
 
 	/* check that we can use the algo */
 	if (have_algo(algo, algolen, sshhostkey) == DROPBEAR_FAILURE) {
-		dropbear_log(LOG_AUTHPRIV | LOG_INFO,
-				"pubkey auth attempt with unknown algo '%s' for '%s'",
-				algo, ses.authstate.username);
+		dropbear_log(LOG_WARNING,
+				"pubkey auth attempt with unknown algo '%s' for '%s' from %s",
+				algo, ses.authstate.printableuser, ses.addrstring);
 		goto out;
 	}
 
 	/* check file permissions */
 	if (checkpubkeyperms() == DROPBEAR_FAILURE) {
 		TRACE(("perms failed"));
-		dropbear_log(LOG_AUTHPRIV | LOG_INFO,
-				"bad authorized_keys permissions for '%s'",
-				ses.authstate.username);
+		dropbear_log(LOG_NOTICE,
+				"bad authorized_keys permissions for '%s' from %s",
+				ses.authstate.printableuser, ses.addrstring);
 		goto out;
 	}
 
 	/* we don't need to check pw and pw_dir for validity, since
 	 * its been done in checkpubkeyperms. */
 	len = strlen(ses.authstate.pw->pw_dir);
-	filename = m_malloc(len + 30);
-	strncpy(filename, ses.authstate.pw->pw_dir, len+1);
+	/* allocate max required pathname storage,
+	 * = path + "/.ssh/authorized_keys" + '\0' = pathlen + 22 */
+	filename = m_malloc(len + 22);
+	strncpy(filename, ses.authstate.pw->pw_dir, len + 1);
 	strncat(filename, "/.ssh/authorized_keys", 21);
 
 	/* open the file */
@@ -333,7 +337,7 @@ static int checkpubkeyperms() {
 
 	/* allocate max required pathname storage,
 	 * = path + "/.ssh/authorized_keys" + '\0' = pathlen + 22 */
-	filename = m_malloc(len + 30);
+	filename = m_malloc(len + 22);
 	strncpy(filename, ses.authstate.pw->pw_dir, len+1);
 
 	/* check ~ */
