@@ -33,6 +33,7 @@
 #include "termcodes.h"
 #include "ssh.h"
 #include "random.h"
+#include "utmp.h"
 
 static int sessioncommand(struct Channel *channel, struct ChanSess *chansess,
 		char iscmd);
@@ -215,6 +216,8 @@ void closechansess(struct Channel *channel) {
 
 	struct ChanSess *chansess;
 	int i;
+	struct logininfo *li;
+
 	chansess = (struct ChanSess*)channel->typedata;
 
 	TRACE(("enter closechansess"));
@@ -225,10 +228,17 @@ void closechansess(struct Channel *channel) {
 
 	m_free(chansess->cmd);
 	m_free(chansess->term);
+
 	if (chansess->tty) {
+		/* write the utmp/wtmp login record */
+		li = login_alloc_entry(chansess->pid, ses.authstate.username,
+				NULL, chansess->tty);
+		login_logout(li);
+		login_free_entry(li);
+
 		pty_release(chansess->tty);
+		m_free(chansess->tty);
 	}
-	m_free(chansess->tty);
 
 	/* clear child pid entries */
 	for (i = 0; i < ses.childpidsize; i++) {
@@ -593,6 +603,7 @@ static int noptycommand(struct Channel *channel, struct ChanSess *chansess) {
 static int ptycommand(struct Channel *channel, struct ChanSess *chansess) {
 
 	pid_t pid;
+	struct logininfo *li;
 	
 
 	TRACE(("enter ptycommand"));
@@ -603,14 +614,13 @@ static int ptycommand(struct Channel *channel, struct ChanSess *chansess) {
 	if (pid < 0)
 		return 1;
 
-	if (!pid) {
+	if (pid == 0) {
 		/* child */
 		
 		/* redirect stdin/stdout/stderr */
 		close(chansess->master);
 
 		pty_make_controlling_tty(&chansess->slave, chansess->tty);
-		m_free(chansess->tty);
 		
 		if (dup2(chansess->slave, STDIN_FILENO) < 0) {
 			TRACE(("leave sessioncommand: error redirecting stdin"));
@@ -629,6 +639,15 @@ static int ptycommand(struct Channel *channel, struct ChanSess *chansess) {
 
 		close(chansess->slave);
 
+		/* write the utmp/wtmp login record - must be after changing the
+		 * terminal used for stdout with the dup2 above */
+		li= login_alloc_entry(getpid(), ses.authstate.username,
+				ses.addrstring, chansess->tty);
+		login_login(li);
+		login_free_entry(li);
+
+		m_free(chansess->tty);
+
 		execchild(chansess);
 		/* not reached */
 
@@ -682,7 +701,9 @@ static void addchildpid(struct ChanSess *chansess, pid_t pid) {
 static void execchild(struct ChanSess *chansess) {
 
 	char *argv[4];
-	int i, len;
+	char * usershell;
+	char * baseshell;
+	int i;
 
 	/* wipe the hostkey */
 	sign_key_free(ses.opts->hostkey);
@@ -698,10 +719,9 @@ static void execchild(struct ChanSess *chansess) {
 		}
 	}
 
-
 	/* clear environment */
 	/* if we're debugging using valgrind etc, we need to keep the LD_PRELOAD
-	 * etc. this is hazardous, so should only be used for debugging. */
+	 * etc. This is hazardous, so should only be used for debugging. */
 #ifndef DEBUG_KEEP_ENV
 #ifdef HAVE_CLEARENV
 	clearenv();
@@ -711,20 +731,30 @@ static void execchild(struct ChanSess *chansess) {
 #endif /* HAVE_CLEARENV */
 #endif /* DEBUG_KEEP_ENV */
 
-	/* change user */
-	if (setgid(ses.authstate.pw->pw_gid) < 0) {
-		dropbear_exit("error changing user");
-	}
-#ifndef HACKCRYPT
-	if (initgroups(ses.authstate.pw->pw_name,
-				ses.authstate.pw->pw_gid) < 0) {
-		dropbear_exit("error changing user");
-	}
-#endif
-	if (setuid(ses.authstate.pw->pw_uid) < 0) {
-		dropbear_exit("error changing user");
-	}
+	/* We can only change uid/gid as root ... */
+	if (getuid() == 0) {
 
+		if (setgid(ses.authstate.pw->pw_gid) < 0
+			|| initgroups(ses.authstate.pw->pw_name,
+				ses.authstate.pw->pw_gid) < 0) {
+			dropbear_exit("error changing user");
+		}
+
+		if (setuid(ses.authstate.pw->pw_uid) < 0) {
+			dropbear_exit("error changing user");
+		}
+	} else {
+		/* ... but if the daemon is the same uid as the requested uid, we don't
+		 * need to */
+
+		/* XXX - there is a minor issue here, in that if there are multiple
+		 * usernames with the same uid, but differing groups, then the
+		 * differing groups won't be set (as with initgroups()). The solution
+		 * is for the sysadmin not to give out the UID twice */
+		if (getuid() != ses.authstate.pw->pw_uid) {
+			dropbear_exit("couldn't	change user as non-root");
+		}
+	}
 
 	/* set env vars */
 	addnewvar("USER", ses.authstate.pw->pw_name);
@@ -740,21 +770,30 @@ static void execchild(struct ChanSess *chansess) {
 		dropbear_exit("error changing directory");
 	}
 
-	/* set up execution */
-	/* shell commandname - the filename portion only */
-	len = strlen(ses.authstate.pw->pw_shell);
-	for (i = len-1; i > 0; i--) {
-		if (ses.authstate.pw->pw_shell[i] == '/') {
-			i++;
-			break;
-		}
+	/* an empty shell should be interpreted as "/bin/sh" */
+	if (ses.authstate.pw->pw_shell[0] == '\0') {
+		usershell = "/bin/sh";
+	} else {
+		usershell = ses.authstate.pw->pw_shell;
 	}
-	argv[0] = &ses.authstate.pw->pw_shell[i];
+
+	baseshell = basename(usershell);
+
+	if (chansess->cmd != NULL) {
+		argv[0] = baseshell;
+	} else {
+		/* a login shell should be "-bash" for "/bin/bash" etc */
+		argv[0] = (char*)m_malloc(strlen(baseshell) + 2); /* 2 for "-" */
+		strcpy(argv[0], "-");
+		strcat(argv[0], baseshell);
+	}
+
 	if (chansess->cmd != NULL) {
 		argv[1] = "-c";
 		argv[2] = chansess->cmd;
 		argv[3] = NULL;
 	} else {
+		/* construct a shell of the form "-bash" etc */
 		argv[1] = NULL;
 	}
 
