@@ -1,3 +1,27 @@
+/*
+ * Dropbear - a SSH2 server
+ * 
+ * Copyright (c) 2002,2003 Matt Johnston
+ * All rights reserved.
+ * 
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ * 
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE. */
+
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
@@ -353,7 +377,8 @@ static void send_msg_kexdh_reply(mp_int *dh_e) {
 
 	/* calculate our random value dh_y */
 	do {
-		genhighrandom(randbuf, dh_q_len);
+		assert(dh_q_len <= sizeof(randbuf));
+		genrandom(randbuf, dh_q_len);
 		if (mp_read_unsigned_bin(&dh_y, randbuf, dh_q_len) != MP_OKAY) {
 			dropbear_exit("Diffie-Hellman error");
 		}
@@ -394,7 +419,7 @@ static void send_msg_kexdh_reply(mp_int *dh_e) {
 	sha1_process(&hs, buf_getptr(ses.kexhashbuf, ses.kexhashbuf->len),
 			ses.kexhashbuf->len);
 	sha1_done(&hs, ses.hash);
-#ifdef DEBUG_KEXHASH
+#if defined(DEBUG_KEXHASH) && defined(DEBUG_TRACE)
 	{
 		int hashfd;
 		int hashwritelen;
@@ -525,7 +550,7 @@ static void read_kex() {
 	/* first_kex_packet_follows */
 	if (buf_getbyte(ses.payload)) {
 		ses.kexstate.firstfollows = 1;
-		dropbear_exit("UNIMPLEMENTED: kexfollows is unhandled"); /* TODO */
+		/* XXX currently not handled */
 	}
 
 	/* reserved for future extensions */
@@ -584,7 +609,7 @@ void send_msg_kexinit() {
 
 	/* set up transmitted kex packet buffer for hashing. 
 	 * This is freed after the end of the kex */
-	ses.transkexinit = buf_newcopy(ses.writepayload, 1);
+	ses.transkexinit = buf_newcopy(ses.writepayload);
 
 	encrypt_packet();
 	ses.dataallowed = 0; /* don't send other packets during kex */
