@@ -20,7 +20,33 @@
  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE. */
+ * SOFTWARE.
+ *
+ * strlcat() is copyright as follows:
+ * Copyright (c) 1998 Todd C. Miller <Todd.Miller@courtesan.com>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. The name of the author may not be used to endorse or promote products
+ *    derived from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
+ * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
+ * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+ * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+ * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
+ * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 
 #include <stdio.h>
 #include <unistd.h>
@@ -48,7 +74,7 @@ void startsyslog() {
 
 	int fd;
 
-	openlog(PROGNAME, LOG_PID, LOG_DAEMON);
+	openlog(PROGNAME, LOG_PID, LOG_AUTHPRIV);
 
 #ifndef DEBUG_TRACE
 	/* redirect stdin/stdout/stderr to /dev/null */
@@ -96,18 +122,14 @@ static void _dropbear_exit(int exitcode, const char* format, va_list param) {
 	char fmtbuf[MAX_FMT + EXIT_MESSAGE_LEN + MAX_INFOSTR + 1];
 	char infostr[MAX_INFOSTR + 1];
 
-#ifdef DOCLEANUP
-	session_cleanup();
-#endif
-
 	strcpy(fmtbuf, EXIT_MESSAGE);
 
 	/* include the username if possible */
 	if (sessinitdone && ses.authstate.authdone) {
 		/* user has authenticated */
 		assert(ses.authstate.username);
-		snprintf(infostr, sizeof(infostr), "post-userauth (%s): ", 
-				ses.authstate.username);
+		snprintf(infostr, sizeof(infostr), "post-userauth (%s) from %s: ", 
+				ses.authstate.printableuser, ses.addrstring);
 	} else {
 		/* before userauth */
 		snprintf(infostr, sizeof(infostr), "before userauth: ");
@@ -116,7 +138,10 @@ static void _dropbear_exit(int exitcode, const char* format, va_list param) {
 	
 	strncat(fmtbuf, format, MAX_FMT);
 
-	_dropbear_log(LOG_DAEMON | LOG_INFO, fmtbuf, param);
+	_dropbear_log(LOG_INFO, fmtbuf, param);
+
+	/* must be after using username etc */
+	session_cleanup();
 
 	exit(exitcode);
 
@@ -135,12 +160,14 @@ void dropbear_log(int priority, const char* format, ...) {
 /* priority is priority as with syslog() */
 static void _dropbear_log(int priority, const char* format, va_list param) {
 
+	char printbuf[1024];
+
+	vsnprintf(printbuf, sizeof(printbuf), format, param);
+
 	if (usingsyslog) {
-		vsyslog(priority, format, param);
+		syslog(priority, printbuf);
 	} else {
-		fprintf(stderr, "dropbear ");
-		vfprintf(stderr, format, param);
-		fprintf(stderr, "\n");
+		fprintf(stderr, "dropbear %s\n", printbuf);
 	}
 
 }
@@ -158,6 +185,65 @@ void dropbear_trace(const char* format, ...) {
 }
 #endif /* DEBUG_TRACE */
 
+/* return a string representation of the socket address passed. Accepts
+ * INET and INET6 types */
+unsigned char * getaddrstring(struct sockaddr * addr) {
+
+	unsigned char * ret;
+
+	/* extensible for ip6 */
+	switch (addr->sa_family) {
+		case PF_INET: 
+			/* max len is "XXX.XXX.XXX.XXX:PPPPP\0" */
+			ret = (unsigned char*)m_malloc(15 + 1);
+			snprintf(ret, 15+1, "%s",
+					inet_ntoa(((struct sockaddr_in *)addr)->sin_addr));
+			return ret;
+			break;
+		default:
+			/* don't get here */
+			assert(0);
+			return NULL;
+			break;
+	}
+
+}
+
+/* get the hostname corresponding to the address in addr, putting the result in
+ * dest, truncating the result to fit in dest. If the result is too long to fit,
+ * the result WON'T be null-terminated.
+ * On failure the string in addr will be copied to dest.
+ * The return value is the length of the string copied to dest */
+int getaddrhostname(char * dest, unsigned int len, char * addr) {
+
+	struct hostent *host = NULL;
+	int retlen;
+	char * retstring;
+
+	if (len < 1) {
+		return 0;
+	}
+
+#ifdef DO_HOST_LOOKUP
+	host = gethostbyname(addr);
+#endif
+	
+	if (!host) {
+		/* return the address */
+		retstring = addr;
+	} else {
+		/* return the hostname */
+		retstring = host->h_name;
+	}
+
+	retlen = strlen(retstring) + 1;
+	retlen = MIN(len, retlen);
+
+	/* we don't need null-termination if it fills the buffer */
+	strncpy(dest, retstring, retlen);
+	return retlen;
+}
+
 #ifndef HAVE_STRLCPY
 /* Implemented by matt as specified in freebsd 4.7 manpage.
  * We don't require great speed, is simply for use with sshpty code */
@@ -184,6 +270,49 @@ size_t strlcpy(char *dst, const char *src, size_t size) {
 }
 #endif /* HAVE_STRLCPY */
 
+#ifndef HAVE_STRNCAT
+/* taken from openbsd-compat for OpenSSH 3.6.1p1 */
+/* "$OpenBSD: strlcat.c,v 1.8 2001/05/13 15:40:15 deraadt Exp $"
+ *
+ * Appends src to string dst of size siz (unlike strncat, siz is the
+ * full size of dst, not space left).  At most siz-1 characters
+ * will be copied.  Always NUL terminates (unless siz <= strlen(dst)).
+ * Returns strlen(src) + MIN(siz, strlen(initial dst)).
+ * If retval >= siz, truncation occurred.
+ */
+size_t
+strlcat(dst, src, siz)
+	char *dst;
+	const char *src;
+	size_t siz;
+{
+	register char *d = dst;
+	register const char *s = src;
+	register size_t n = siz;
+	size_t dlen;
+
+	/* Find the end of dst and adjust bytes left but don't go past end */
+	while (n-- != 0 && *d != '\0')
+		d++;
+	dlen = d - dst;
+	n = siz - dlen;
+
+	if (n == 0)
+		return(dlen + strlen(s));
+	while (*s != '\0') {
+		if (n != 1) {
+			*d++ = *s;
+			n--;
+		}
+		s++;
+	}
+	*d = '\0';
+
+	return(dlen + (s - src));	/* count does not include NUL */
+}
+#endif /* HAVE_STRLCAT */
+
+#ifdef DEBUG_TRACE
 void printhex(unsigned char* buf, int len) {
 
 	int i;
@@ -199,6 +328,35 @@ void printhex(unsigned char* buf, int len) {
 	}
 	fprintf(stderr, "\n");
 }
+#endif
+
+/* Strip all control characters from text (a null-terminated string), except
+ * for '\n', '\r' and '\t'.
+ * The result returned is a newly allocated string, this must be free()d after
+ * use */
+char * stripcontrol(const char * text) {
+
+	char * ret;
+	int len, pos;
+	int i;
+	
+	len = strlen(text);
+	ret = m_malloc(len+1);
+
+	pos = 0;
+	for (i = 0; i < len; i++) {
+		if ((text[i] <= '~' && text[i] >= ' ') /* normal printable range */
+				|| text[i] == '\n' || text[i] == '\r' || text[i] == '\t') {
+			ret[pos] = text[i];
+			pos++;
+		}
+	}
+	ret[pos] = 0x0;
+	return ret;
+}
+			
+
+
 
 /* returns the current position on success, or -1 on failure */
 int readln(int fd, char* buf, int count) {
